""" Preprocessing algorithms for UFL forms """

import ufl.classes as uc

from pytools import memoize


@memoize
def preprocess_form(form):
    from ufl.algorithms import compute_form_data
    formdata = compute_form_data(form,
                                 do_apply_function_pullbacks=True,
                                 do_apply_geometry_lowering=True,
                                 do_apply_integral_scaling=True,
                                 preserve_geometry_types=(uc.CellVolume,
                                                          uc.FacetArea,
                                                          uc.FacetJacobianDeterminant,
                                                          uc.FacetNormal,
                                                          uc.JacobianDeterminant,
                                                          uc.JacobianInverse,
                                                          )
                                 )

    formdata.preprocessed_form = apply_default_transformations(formdata.preprocessed_form)

    return formdata


def apply_default_transformations(form):
    #
    # This is the list of transformations we unconditionally apply to
    # all forms we want to generate code for.
    #
    from dune.perftool.ufl.transformations import transform_form
    from dune.perftool.ufl.transformations.indexpushdown import pushdown_indexed

    form = transform_form(form, pushdown_indexed)

    return form
